//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#pragma once
#include "types.h"
#include "opcodes.h"
#include "otherfunctions.h"
#include "MD5Sum.h"
#include <string>
#include "securedvars.h"

using namespace std;

const CString strDefaultToolbar = _T("0099010203040506079908");

#pragma pack(1)
// DO NOT EDIT VALUES like making a uint16 to uint32, or insert any value. ONLY append new vars
struct Preferences_Ext_Struct{
	int8	version;
	uchar	userhash[16];
	WINDOWPLACEMENT EmuleWindowPlacement;
};
// Maverick
struct ProxySettings{
	uint16 type;
	uint16 port;
	char name[50];
	char user[50];
	char password[50];
	bool EnablePassword;
	bool UseProxy;
};



typedef enum
{
	SEE_SHARE_EVERYBODY = 0,
	SEE_SHARE_FRIENDS = 1,
	SEE_SHARE_NOONE = 2
} enSeeShare;

typedef enum
{
	HASH_PRIORITY_IDLE = 0,
	HASH_PRIORITY_LOWER = 1,
	HASH_PRIORITY_STANDARD = 2
} enHashPriority;

struct Preferences_Struct_DONTUSE{
	uchar	userhash[16];
	WINDOWPLACEMENT EmuleWindowPlacement;
	int16	downloadColumnWidths[13];
	BOOL	downloadColumnHidden[13];
	INT		downloadColumnOrder[13];
	int16	uploadColumnWidths[10];
	BOOL	uploadColumnHidden[10];
	INT		uploadColumnOrder[10];
	int16	queueColumnWidths[13];
	BOOL	queueColumnHidden[13];
	INT		queueColumnOrder[13];
	int16	searchColumnWidths[5];
	BOOL	searchColumnHidden[5];
	INT		searchColumnOrder[5];
	int16	sharedColumnWidths[11];
	BOOL	sharedColumnHidden[11];
	INT		sharedColumnOrder[11];
	int16	serverColumnWidths[9];
	BOOL	serverColumnHidden[9];
	INT 	serverColumnOrder[9];
	int16	ircColumnWidths[3];
	BOOL	ircColumnHidden[3];
	INT		ircColumnOrder[3];

	uint8	flatbar;

	uint32 slsmaxsourcesperfile;

	bool	BugReport;

	bool	m_bpreviewprio;

	// Tarod
	uchar	A4AF_file_hash[16];

	bool	m_bmanualhighprio;
	int8	globalsearch; // Barry

	// Barry - Provide a mechanism for all tables to store/retrieve sort order
	int		tableSortItemDownload;
	int		tableSortItemUpload;
	int		tableSortItemQueue;
	int		tableSortItemSearch;
	int		tableSortItemShared;
	int		tableSortItemServer;
	bool	tableSortAscendingDownload;
	bool	tableSortAscendingUpload;
	bool	tableSortAscendingQueue;
	bool	tableSortAscendingSearch;
	bool	tableSortAscendingShared;
	bool	tableSortAscendingServer;
	char	TxtEditor[256];
	bool	showRatesInTitle;
	CString	messageFilter;
	uint16	maxmsgsessions;

	ProxySettings proxy; // deadlake PROXYSUPPORT
	bool		m_bIsASCWOP;
};

struct Preferences_Import19c_Struct{
	int8	version;
	char	nick[50];
	uint16	maxupload;
	uint16	maxdownload;
	uint16	port;
	uint16	maxconnections;
	int8	reconnect;
	int8	deadserver;
	int8	scorsystem;
	char	incomingdir[510];
	char	tempdir[510];
	int8	ICH;
	int8	autoserverlist;
	int8	updatenotify;
	int8	mintotray;
	uchar	userhash[16];
	int8	autoconnect;
	int8	addServersFromConn;
};

struct Preferences_Import20a_Struct{
	int8	version;
	char	nick[50];
	uint16	maxupload;
	uint16	maxdownload;
	uint16	port;
	uint16	maxconnections;
	int8	reconnect;
	int8	deadserver;
	uint16	deadserverretries;
	int8	scorsystem;
	char	incomingdir[510];
	char	tempdir[510];
	int8	ICH;
	int8	autoserverlist;
	int8	updatenotify;
	int8	mintotray;
	uchar	userhash[16];
	int8	autoconnect;
	int8	addServersFromConn;
	int8	addserversfromclient;
	int16	maxsourceperfile;
	int16	trafficOMeterInterval;
	int32   totalDownloaded;
	int32	totalUploaded;
	int	maxGraphDownloadRate;
	int	maxGraphUploadRate;
	uint8	beepOnError;
	uint8	confirmExit;
	WINDOWPLACEMENT EmuleWindowPlacement;
	int transferColumnWidths[9];
	int serverColumnWidths[8];
	uint8	splashscreen;
	uint8	filerBadIP;
};

struct Preferences_Import20b_Struct{
	int8	version;
	char	nick[50];
	uint16	maxupload;
	uint16	maxdownload;
	uint16	port;
	uint16	maxconnections;
	int8	reconnect;
	int8	deadserver;
	int8	scorsystem;
	char	incomingdir[510];
	char	tempdir[510];
	int8	ICH;
	int8	autoserverlist;
	int8	updatenotify;
	int8	mintotray;
	uchar	userhash[16];
	int8	autoconnect;
	int8	addServersFromConn;
	int8	addserversfromclient;
	int16	maxsourceperfile;
	int16	trafficOMeterInterval;
	int32   totalDownloaded;	// outdated
	int32	totalUploaded;		// outdated
	int		maxGraphDownloadRate;
	int		maxGraphUploadRate;
	uint8	beepOnError;
	uint8	confirmExit;
	WINDOWPLACEMENT EmuleWindowPlacement;
	int transferColumnWidths[9];
	int serverColumnWidths[8];
	uint8	splashscreen;
	uint8	filerBadIP;
	int64   totalDownloadedBytes;
	int64	totalUploadedBytes;
};

class CPreferences{
public:
	enum Table {tableDownload, tableUpload, tableQueue, tableSearch, tableShared, tableServer, tableIrc};

	CPreferences();
	~CPreferences();

	char*	GetAppDir()									{return appdir;}
	bool	Save();

	uint16	Profile()									{ return m_nProfile.Get(); }
	void	SetProfile(uint16 uProfile)					{ m_nProfile.Put(uProfile); }
	int8	Score()										{ return m_nScoreSystem.Get(); }
	void	SetScore(int8 iScore)						{ m_nScoreSystem.Put(iScore); }
	bool	Reconnect()									{ return m_bReconnect.Get(); }
	void	SetReconnect(bool bReconnect)				{ m_bReconnect.Put(bReconnect); }
	int8	DeadServer()								{ return m_nDeadServer.Get(); }
	void	SetDeadServer(int8 iDead)					{ m_nDeadServer.Put(iDead); }
	CString	GetUserNick()								{ return CString(m_sNick.Get().c_str()); }
	void    SetUserNick(CString strNewNick)				{ m_sNick.Put(strNewNick.GetBuffer(0)); }
	uint16	GetPort()									{ return m_nPort.Get(); }
	void	SetPort(uint16 uPort)						{ m_nPort.Put(uPort); }
	
    //Maverick
    uint16	GetListenPort()								{ if(m_bUseProxyListenPort.Get()) return m_nListenPort.Get(); else return GetPort(); }
	void	SetListenPort(uint16 uPort)					{ m_nListenPort.Put(uPort); m_bUseProxyListenPort.Put(true); }
	void	ResetListenPort()							{ m_nListenPort.Put(0); m_bUseProxyListenPort.Put(false); }
	
	uint16	GetUDPPort()								{ return m_nUDPPort.Get(); }
	void	SetUDPPort(uint16 uPort)					{ m_nUDPPort.Put(uPort); }
	CString	GetIncomingDir()							{ return CString(m_sIncomingDir.Get().c_str()); }
	void	SetIncomingDir(CString strDir)				{ m_sIncomingDir.Put(strDir.GetBuffer(0)); }
	CString	GetTempDir()								{ return CString(m_sTempDir.Get().c_str()); }
	void	SetTempDir(CString strDir)					{ m_sTempDir.Put(strDir.GetBuffer(0)); }
	CString	GetVlcDir()  								{ return CString(m_sVlcDir.Get().c_str()); }		//Cax2 - keep for compatibility in 1a...
	//void    SetVlcDir(CString strDir)					{ m_sVlcDir.Put(strDir.GetBuffer(0)); }			//Cax2 - not needed anymore
	CString	GetVideoPlayer()  							{ return CString(m_sVideoPlayer.Get().c_str()); }
	void    SetVideoPlayer(CString str)					{ m_sVideoPlayer.Put(str.GetBuffer(0)); }
	uchar*	GetUserHash()								{ return prefs->userhash; }
	uint16	GetMaxUpload()								{ return m_nMaxUpload.Get(); }

	// Added by Tarod
	bool	IsUAPEnabled()								{ return m_bUAP.Get(); }
	void	SetUAPEnabled(bool bEnable)					{ m_bUAP.Put(bEnable); }
	bool	IsDAPEnabled()								{ return m_bDAP.Get(); }
	void	SetDAPEnabled(bool bEnable)					{ m_bDAP.Put(bEnable); }
//	bool	ReconnectLowID() 							{ return prefs->reconnectlowid;}
	bool	IsDisabledXS()								{ return m_bDisableXS.Get(); }
	void	SetDisabledXS(bool bDisable)				{ m_bDisableXS.Put(bDisable); }
	bool	StartDownloadPaused()						{ return m_bStartDownloadPaused.Get(); }
	void	SetStartDownloadPaused(bool bPaused)		{ m_bStartDownloadPaused.Put(bPaused); }
	bool	DownloadPausedOnComplete()					{ return m_bDownloadPausedOnComplete.Get(); }
	void	SetDownloadPausedOnComplete(bool bPaused)	{ m_bDownloadPausedOnComplete.Put(bPaused); }
	bool	IsRemoveNNSEnabled() 						{ return m_bRemoveNNS.Get(); }
	void	SetRemoveNNSEnabled(bool bEnable)			{ m_bRemoveNNS.Put(bEnable); }
	CString GetToolbarSettings() 						{ return CString(m_sToolbarSettings.Get().c_str()); }
	void	SetToolbarSettings(TCHAR* in) 				{ m_sToolbarSettings.Put(in); }
	CString GetToolbarBitmapSettings() 					{ return CString(m_sToolbarBitmap.Get().c_str()); } // reCDVst
	void	SetToolbarBitmapSettings(CString path) 		{ m_sToolbarBitmap.Put(path.GetBuffer(0)); } // reCDVst
	CString GetToolbarBitmapFolderSettings() 			{ return CString(m_sToolbarBitmapFolder.Get().c_str()); } // reCDVst
	void	SetToolbarBitmapFolderSettings(CString path){ m_sToolbarBitmapFolder.Put(path.GetBuffer(0)); } // reCDVst
	int8	GetToolbarLabelSettings()					{ return m_nToolbarLabels.Get(); }
	void	SetToolbarLabelSettings(int8 settings)		{ m_nToolbarLabels.Put(settings); }
	bool	AutoServerlist()							{ return m_bAutoServerList.Get(); }
	void	SetAutoServerlist(bool bAuto)				{ m_bAutoServerList.Put(bAuto); }
	uchar*	GetA4AFHash(void)							{ return prefs->A4AF_file_hash; }
	void	SetA4AFHash(uchar* file_hash)				{ memcpy(prefs->A4AF_file_hash, file_hash, sizeof(prefs->A4AF_file_hash)); }

	// WebServer
	uint16	GetWSPort()									{ return m_nWebPort.Get(); }
	void	SetWSPort(uint16 uPort)						{ m_nWebPort.Put(uPort); }
	CString	GetWSPass()									{ return CString(m_sWebPassword.Get().c_str()); }
	void	SetWSPass(CString strNewPass)				{ m_sWebPassword.Put(MD5Sum(strNewPass).GetHash().GetBuffer(0)); }
	CString	GetWSLowPass()								{ return CString(m_sLowWebPassword.Get().c_str()); }
	void	SetWSLowPass(CString strNewPass)			{ m_sLowWebPassword.Put(MD5Sum(strNewPass).GetHash().GetBuffer(0)); }
	bool	GetWSIsEnabled()							{ return m_bWebEnabled.Get(); }
	void	SetWSIsEnabled(bool bEnable)				{ m_bWebEnabled.Put(bEnable); }
	bool	GetWSIsLowUserEnabled()						{ return m_bWebLowEnabled.Get(); }
	void	SetWSIsLowUserEnabled(bool bEnable)			{ m_bWebLowEnabled.Put(bEnable); }
	bool	GetWebUseGzip()								{ return m_bWebUseGzip.Get(); }
	void	SetWebUseGzip(bool bUse)					{ m_bWebUseGzip.Put(bUse); }
	int		GetWebPageRefresh()							{ return m_nWebPageRefresh.Get(); }
	void	SetWebPageRefresh(int nRefresh)				{ m_nWebPageRefresh.Put(nRefresh); }

	bool	UpdateNotify()								{ return m_bUpdateNotify.Get(); }
	bool	DoMinToTray()								{ return m_bMinToTray.Get(); }
	void	SetMinToTray(bool bMinToTray)				{ m_bMinToTray.Put(bMinToTray); }
	bool	DoAutoConnect() 							{ return m_bAutoConnect.Get(); }
	void    SetAutoConnect(bool bConnect)				{ m_bAutoConnect.Put(bConnect); }
	uint32	BadClientBanTime()							{ return m_nBadClientBanTime.Get(); }
	void	SetBadClientBanTime(uint32 uTime)			{ m_nBadClientBanTime.Put(uTime); }
	uint32	BadClientMinRequestTime()					{ return m_nMinRequestTime.Get(); }
	void	SetBadClientMinRequestTime(uint32 uTime)	{ m_nMinRequestTime.Put(uTime); }
	uint8	BadClientMinRequestNum()					{ return m_nBadClientBan.Get(); }
	void	SetBadClientMinRequestNum(uint8 uVal)		{ m_nBadClientBan.Put(uVal); }

	// reCDVst (Load/Save Sources settings)
	uint32	RestoreSourcesTime()						{ return m_nSLSRestoreSourcesTime.Get(); }
	void	SetRestoreSourcesTime(uint32 uTime)			{ m_nSLSRestoreSourcesTime.Put(uTime); }
	uint32	LoadQuantities()							{ return m_nSLSLoadQuantities.Get(); }
	void 	SetLoadQuantities(uint32 uVal)				{ m_nSLSLoadQuantities.Put(uVal); }
	uint32	WhenSourcesOutdated()						{ return m_nSLSOutdated.Get(); }
	void	SetWhenSourcesOutdated(uint32 uVal)			{ m_nSLSOutdated.Put(uVal);}
	uint32	MaxSourcesPerFile()							{ return prefs->slsmaxsourcesperfile; }
	bool	SLSEnable()									{ return m_bSLSEnable.Get(); }
	void	SetSLSEnable(bool bEnable)					{ m_bSLSEnable.Put(bEnable); }

	int16	GetQueueSize()								{ return m_nQueueSize.Get(); }
	void	SetQueueSize(int16 iSize)					{ m_nQueueSize.Put(iSize); }
	int16	GetRemoteQueueLimit()						{ return m_nRQlimit.Get(); }
	void	SetRemoteQueueLimit(uint16 uVal)			{ m_nRQlimit.Put(uVal); }
	bool	UseRemoteQueueLimit()						{ return m_bUseRQlimit.Get(); }		//Cax2 23/02/03
	void	UseRemoteQueueLimit(bool bit)				{ m_bUseRQlimit.Put(bit); }
	
	bool	GetAddServersFromConn()						{ return m_bAddServersFromConn.Get(); }
	void	SetAddServersFromConn(bool bAdd)			{ m_bAddServersFromConn.Put(bAdd); }
	//bool	AddServersFromClient()						{CSingleLock PrefLock(&m_Pref_Mutex,TRUE); return prefs->addserversfromclient;}
	//void	SetAddServersFromClient(bool bAdd)			{CSingleLock PrefLock(&m_Pref_Mutex,TRUE); prefs->addserversfromclient	=	bAdd;}
//	int8*	GetMinTrayPTR() 							{CSingleLock PrefLock(&m_Pref_Mutex,TRUE); return &prefs->mintotray;}
	uint16	GetTrafficOMeterInterval() 					{ return m_nTrafficOMeterInterval.Get(); }
	void	SetTrafficOMeterInterval(uint16 in) 		{ m_nTrafficOMeterInterval.Put(in); }
	uint16	GetStatsInterval() 							{ return m_nStatsInterval.Get(); }
	void	SetStatsInterval(uint16 in) 				{ m_nStatsInterval.Put(in); }
	uint16  GetStatsColumnWidth()						{ return m_nStatsColumnWidth.Get(); }
	uint16  GetDetailColumnWidth()						{ return m_nDetailColumnWidth.Get(); }
	void    SetStatsColumnWidth(uint16 in)				{ m_nStatsColumnWidth.Put(in); }
	void    SetDetailColumnWidth(uint16 in)				{ m_nDetailColumnWidth.Put(in); }
	CString	GetStatsListExpanded()						{ return CString(m_sStatsListExpanded.Get().c_str()); }
	void	SetStatsListExpanded(CString in)			{ m_sStatsListExpanded.Put(in.GetBuffer(0)); }
	void	Add2TotalDownloaded(uint64 in)				{ uint64 nSet = m_nTotalDownloadedBytes.Get(); nSet += in; m_nTotalDownloadedBytes.Put(nSet); }
	uint64  GetTotalDownloaded()						{ return m_nTotalDownloadedBytes.Get(); }
	void	Add2TotalUploaded(uint64 in) 				{ uint64 nSet = m_nTotalUploadedBytes.Get(); nSet += in; m_nTotalUploadedBytes.Put(nSet); }
	uint64	GetTotalUploaded()							{ return m_nTotalUploadedBytes.Get(); }
	bool	IsErrorBeepEnabled()						{ return m_bBeepOnError.Get(); }
	void	SetErrorBeepEnabled(bool bEnable)			{ m_bBeepOnError.Put(bEnable); }
	bool	IsConfirmExitEnabled()						{ return m_bConfirmExit.Get(); }
	void	SetConfirmExitEnabled(bool bEnable)			{ m_bConfirmExit.Put(bEnable); }
	bool	UseSplashScreen()							{ return m_bSplashScreen.Get(); }
	void	SetUseSplashScreen(bool bUse)				{ m_bSplashScreen.Put(bUse); }
	bool	FilterBadIPs()								{ return m_bFilterBadIP.Get(); }
	void    SetFilterBadIPs(bool bFilter)				{ m_bFilterBadIP.Put(bFilter); }
	bool	IsOnlineSignatureEnabled()  				{ return m_bOnlineSig.Get(); }
	void	SetOnlineSignatureEnabled(bool bEnable)		{ m_bOnlineSig.Put(bEnable); }

	int		GetMaxGraphUploadRate()						{ return m_nMaxGraphUploadRate.Get(); }
	void	SetMaxGraphUploadRate(int in)				{ m_nMaxGraphUploadRate.Put(in); }
	int		GetMaxGraphDownloadRate()					{ return m_nMaxGraphDownloadRate.Get(); }
	void	SetMaxGraphDownloadRate(int in)				{ m_nMaxGraphDownloadRate.Put(in); }

	uint16	GetMaxDownload();
	uint16	GetMaxConnections()							{ return m_nMaxConnections.Get(); }
	void	SetMaxConnections(uint16 uMaxConn)			{ m_nMaxConnections.Put(uMaxConn); }
	uint16	GetMaxSourcePerFile()						{ return m_nMaxSourcePerFile.Get(); }
	void	SetMaxSourcePerFile(uint16 uVal)			{ m_nMaxSourcePerFile.Put(uVal); }
	uint16	GetMaxSourcePerFileSoft()					{ uint16 temp = m_nMaxSourcePerFile.Get()*0.9;  return (temp > 1000)?1000:temp; }
	//Cax2 - now calculated via the function above...
	//uint16  GetMaxSourcesPerFileSoftValue()			{CSingleLock PrefLock(&m_Pref_Mutex_MaxSourcesPerFile,TRUE); uint16 nRet = prefs->maxsourceperfilesoft; return nRet; }
	//void    SetMaxSourcesPerFileSoftValue(uint16 uSrc) {CSingleLock PrefLock(&m_Pref_Mutex_MaxSourcesPerFile,TRUE); prefs->maxsourceperfilesoft = uSrc; }
	uint16	GetMaxSourcePerFileUDP()					{ uint16 temp = m_nMaxSourcePerFile.Get()*0.75; return (temp > 100)? 100:temp; }
	uint16	GetDeadserverRetries()						{ return m_nDeadServerRetries.Get(); }
	void	SetDeadserverRetries(uint16 uRetries)		{ m_nDeadServerRetries.Put(uRetries); }

	int     GetColumnWidth (Table t, int index) const;
	BOOL    GetColumnHidden(Table t, int index) const;
	int     GetColumnOrder (Table t, int index) const;
	void	SetColumnWidth (Table t, int index, int width);
	void	SetColumnHidden(Table t, int index, BOOL bHidden);
	void	SetColumnOrder (Table t, INT *piOrder);

	// Barry - Provide a mechanism for all tables to store/retrieve sort order
	int		GetColumnSortItem (Table t) const;
	bool	GetColumnSortAscending (Table t) const;
	void	SetColumnSortItem (Table t, int sortItem);
	void	SetColumnSortAscending (Table t, bool sortAscending);



	bool	GetMultiple()								{ return m_bMultiple.Get(); }
	void	SetMultiple(bool bEnable)					{ m_bMultiple.Put(bEnable); }

	WORD	GetLanguageID()								{ return m_nLanguageID.Get(); }
	void	SetLanguageID(WORD wID)						{ m_nLanguageID.Put(wID); }
	int8	IsDoubleClickEnabled()						{ return m_nTransferDoubleclick.Get(); }
	void	SetDoubleClickEnabled(int8 iEnable)			{ m_nTransferDoubleclick.Put(iEnable); }
	enSeeShare CanSeeShares(void)						{ return m_nSeeShares.Get(); }
	void	SetCanSeeShares(enSeeShare iCan)			{ m_nSeeShares.Put(iCan); }
	int8	GetToolTipDelay(void)						{ return m_nTooltipDelayTime.Get(); }
	void	SetToolTipDelay(int8 iDelay)				{ m_nTooltipDelayTime.Put(iDelay); }
	bool	IsBringToFront()							{ return m_bBringToForeground.Get(); }
	void	SetBringToFront(bool bBTFront)				{ m_bBringToForeground.Put(bBTFront); }

	int8    GetSplitterbarPosition()					{ return m_nSplitterBarPosition.Get(); }
	void	SetSplitterbarPosition(int8 pos) 			{ m_nSplitterBarPosition.Put(pos); }
	uint16	GetStatsMax()								{ return m_nStatsMax.Get(); }
	void	SetStatsMax(uint16 uMaxStats)				{ m_nStatsMax.Put(uMaxStats); }
	int8	GetStatsAverageMinutes()					{ return m_nStatsAverageMinutes.Get(); }
	void	SetStatsAverageMinutes(int8 in)				{ m_nStatsAverageMinutes.Put(in); }

	bool	GetShowToolbarSpeedMeter()					{ return m_bShowToolbarSpeedMeter.Get(); }
	void	SetShowToolbarSpeedMeter(bool bShow)		{ m_bShowToolbarSpeedMeter.Put(bShow); }
	bool    GetCloseToTray()							{ return m_bCloseToTray.Get(); }
	void	SetCloseToTray(bool bClose)					{ m_bCloseToTray.Put(bClose); }
	bool    GetDetailsOnClick()							{ return m_bDetailsOnClick.Get(); }
	void    SetDetailsOnClick(bool bSet)				{ m_bDetailsOnClick.Put(bSet); }
	bool    GetBugReport()								{ return prefs->BugReport; }

	bool    GetUseDownloadAddNotifier()					{ return m_bUseDownloadAddNotifier.Get(); }
	void    SetUseDownloadAddNotifier(bool bUse)		{ m_bUseDownloadAddNotifier.Put(bUse); }
	bool    GetUseDownloadNotifier() 					{ return m_bUseDownloadNotifier.Get(); }
	void    SetUseDownloadNotifier(bool bUse)			{ m_bUseDownloadNotifier.Put(bUse); }
	bool    GetUseChatNotifier()						{ return m_bUseChatNotifier.Get(); }
	void    SetUseChatNotifier(bool bUse)				{ m_bUseChatNotifier.Put(bUse); }
	bool    GetUseLogNotifier()			 				{ return m_bUseLogNotifier.Get(); }
	void    SetUseLogNotifier(bool bUse) 				{ m_bUseLogNotifier.Put(bUse); }
	bool    GetUseSoundInNotifier()		 				{ return m_bUseSoundInNotifier.Get(); }
	void    SetUseSoundInNotifier(bool bUse)			{ m_bUseSoundInNotifier.Put(bUse); }
	bool    GetNotifierPopsEveryChatMsg()				{ return m_bNotifierPopsEveryChatMsg.Get(); }
	void    SetNotifierPopsEveryChatMsg(bool bPop)		{ m_bNotifierPopsEveryChatMsg.Put(bPop); }
	bool	GetNotifierPopOnImportantError()			{ return m_bNotifierImportantError.Get(); }
	void	SetNotifierPopOnImportantError(bool bPop)	{ m_bNotifierImportantError.Put(bPop); }
	CString GetNotifierWavSoundPath() 					{ return CString(m_sNotifierSoundFilePath.Get().c_str()); }
	void	SetNotifierWavSoundPath(CString strPath)	{ m_sNotifierSoundFilePath.Put(strPath.GetBuffer(0)); }

	CString	GetIRCNick()								{ return CString(m_sIRCNick.Get().c_str()); }
	void	SetIRCNick( char in_nick[] )				{ m_sIRCNick.Put(in_nick); }
	CString	GetIRCServer()								{ return CString(m_sIRCServer.Get().c_str()); }
	void	SetIRCServer( char in_serv[] )				{ m_sIRCServer.Put(in_serv); }
	bool	GetIRCAddTimestamp()						{ return m_bIRCAddTimestamp.Get(); }
	void	SetIRCAddTimestamp( bool flag )				{ m_bIRCAddTimestamp.Put(flag); }
	CString	GetIRCChanNameFilter()						{ return CString(m_sIRCChanNameFilter.Get().c_str()); }
	void	SetIRCChanNameFilter(char in_name[])		{ m_sIRCChanNameFilter.Put(in_name); }
	bool	GetIRCUseChanFilter()						{ return m_bIRCUseChanFilter.Get(); }
	void	SetIRCUseChanFilter(bool flag)				{ m_bIRCUseChanFilter.Put(flag); }
	uint16	GetIRCChannelUserFilter()					{ return m_nIRCChannelUserFilter.Get(); }
	void	SetIRCChanUserFilter(uint16 in_user)		{ m_nIRCChannelUserFilter.Put(in_user); }
	CString	GetIrcPerformString()						{ return CString(m_sIRCPerformString.Get().c_str()); }
	void	SetIRCPerformString(char in_perf[])			{ m_sIRCPerformString.Put(in_perf);}
	bool	GetIrcUsePerform()							{ return m_bIRCUsePerform.Get(); }
	void	SetIrcUsePerform(bool flag)					{ m_bIRCUsePerform.Put(flag); }
	bool	GetIRCListOnConnect()						{ return m_bIRCListOnConnect.Get(); }
	void	SetIRCListonConnect(bool flag)				{ m_bIRCListOnConnect.Put(flag); }
	bool	GetIrcAcceptLinks()							{ return m_bIRCAcceptLinks.Get(); }
	void	SetIrcAcceptLInks(bool flag)				{ m_bIRCAcceptLinks.Put(flag); }
	bool	GetIrcIgnoreInfoMessage()					{ return m_bIRCIgnoreInfoMessage.Get(); }
	void	SetIrcIgnoreInfoMessage(bool flag)			{ m_bIRCIgnoreInfoMessage.Put(flag); }
	bool	GetIrcIgnoreEmuleProtoInfoMessage()			{ return m_bIRCIgnoreEmuleProtoInfoMessage.Get(); }
	void	SetIrcIgnoreEmuleProtoInfoMessage(bool flag){ m_bIRCIgnoreEmuleProtoInfoMessage.Put(flag);}
	WORD	GetWindowsVersion();
	bool	GetStartMinimized()							{ return m_bStartMinimized.Get(); }
	void    SetStartMinimized(bool bStart)				{ m_bStartMinimized.Put(bStart); }
	bool	GetSmartIdCheck()							{ return m_bSmartIdCheck.Get(); }
	void	SetSmartIdCheck(bool in_smartidcheck)		{ m_bSmartIdCheck.Put(in_smartidcheck); }
	uint8	GetSmartIdState()							{ return m_nSmartIdState.Get(); }
	void	SetSmartIdState(uint8 in_smartidstate)		{ m_nSmartIdState.Put(in_smartidstate); }

	bool	GetVerbose()						        { return m_bVerbose.Get(); }
	void 	SetVerbose(bool bEnabled)			        { m_bVerbose.Put(bEnabled); }
	bool	GetPreviewPrio()					        { return prefs->m_bpreviewprio; }
	void	SetPreviewPrio(bool bPreview)				{ prefs->m_bpreviewprio = bPreview; }
	bool	GetUpdateQueueList()				        { return m_bUpdateQueueList.Get(); }
	void	SetUpdateQueueList(bool bEnabled)			{ m_bUpdateQueueList.Put(bEnabled); }
	bool	GetManualHighPrio()					        { return prefs->m_bmanualhighprio;}
	bool	ShowOverhead()						        { return m_bShowOverhead.Get();}
	void	SetShowOverhead(bool bit)					{ m_bShowOverhead.Put(bit);}
	bool	BackupPreview()								{return m_bBackupPreview.Get();}
	void	SetBackupPreview(bool bit)					{ m_bBackupPreview.Put(bit);}

	char*	GetTxtEditor()						        {return prefs->TxtEditor;}

	// Barry
	int8	UseFlatBar()								{ return m_nDepth3D.Get()==0; }
	int8	Get3DDepth()                                { return m_nDepth3D.Get(); }
	void	Set3DDepth(int8 iStyle)                      { m_nDepth3D.Put(iStyle); }
//	bool	AddNewFilesPaused()                         {CSingleLock PrefLock(&m_Pref_Mutex,TRUE); return prefs->addnewfilespaused;}
	bool    AutoConnectStaticOnly()                   	{return m_bAutoConnectStaticOnly.Get();}	
	void    SetAutoConnectStaticOnly(bool bEnabled)		{m_bAutoConnectStaticOnly.Put(bEnabled);}	

	DWORD	GetStatsColor(int index) 					{ return m_nStatColors[index].Get(); }
	void	SetStatsColor(int index, DWORD value) 		{ m_nStatColors[index].Put(value); }
	uint8	GetGraphRatio()								{ return m_nGraphRatio.Get(); }
	void	SetGraphRatio(uint8 iNum)					{ m_nGraphRatio.Put(iNum); }
	bool	DoUseSort()									{ return m_bUseSort.Get(); }
	void    SetUseSort(bool bUseSort)					{ m_bUseSort.Put(bUseSort); }
	bool	DoUseSrcSortCol2()							{ return m_bUseSrcSortCol2.Get(); }
	void	SetUseSrcSortCol2(bool bUseSort)			{ m_bUseSrcSortCol2.Put(bUseSort); }
	int8	GetServerSortCol()							{ return m_nServerSortCol.Get(); }
	void	SetServerSortCol(int8 iCol)					{ m_nServerSortCol.Put(iCol); }
	int8	GetUploadSortCol()							{ return m_nUploadSortCol.Get(); }
	void	SetUploadSortCol(int8 iSortCol)				{ m_nUploadSortCol.Put(iSortCol); }
	int8	GetQueueSortCol()							{ return m_nQueueSortCol.Get(); }
	void	SetQueueSortCol(int8 iSortCol)				{ m_nQueueSortCol.Put(iSortCol); }
	int8	GetSearchSortCol()							{ return m_nSearchSortCol.Get(); }
	void	SetSearchSortCol(int8 iSortCol)				{ m_nSearchSortCol.Put(iSortCol); }
	int8	GetIrcSortCol()								{ return m_nIRCSortCol.Get(); }
	void	SetIrcSortCol(int8 iSortCol)				{ m_nIRCSortCol.Put(iSortCol); }
	uint16	GetFileSortCol()							{ return m_nFileSortCol.Get(); }
	void	SetFileSortCol(uint16 uCol)					{ m_nFileSortCol.Put(uCol); }
	int8	GetDownloadSortCol()						{ return m_nDownloadSortCol.Get(); }
	void	SetDownloadSortCol(int8 iSortCol)			{ m_nDownloadSortCol.Put(iSortCol); }
	int8	GetSrcSortCol1()							{ return m_nSrcSortCol1.Get(); }
	void	SetSrcSortCol1(int8 iSortCol)				{ m_nSrcSortCol1.Put(iSortCol); }
	int8	GetSrcSortCol2()							{ return m_nSrcSortCol2.Get(); }
	void	SetSrcSortCol2(int8 iSortCol)				{ m_nSrcSortCol2.Put(iSortCol); }
	
	void	SetMaxDownloadConperFive(int in)			{ m_nMaxConperFive.Put(in); }
	uint16	GetMaxConperFive()							{ return m_nMaxConperFive.Get(); }
	uint16	GetDefaultMaxConperFive();

	//void	ResetStatsColor(int index); //Cax2 changed this function to DefaultStatsColor to use it in the colour picker!
	DWORD	DefaultStatsColor(int index);
	bool	IsSafeConnectEnabled()						{ return m_bSafeConnect.Get(); }
	void	SetSafeConnectEnabled(bool in)				{ m_bSafeConnect.Put(in); }

	//--- xrmb:partTrafficPrefs ---
	bool	DoUsePT()									{ return m_bUsePartTraffic.Get(); }
	void	SetUsePT(bool in)							{ m_bUsePartTraffic.Put(in); }
	uint8	GetUpbarStyle()								{ return m_nUpbarPartTraffic.Get(); }
	void	SetUpbarStyle(uint8 uStyle)					{ m_nUpbarPartTraffic.Put(uStyle); }
	uint8	GetUpbarColor()								{ return m_nUpbarColorPartTraffic.Get(); }
	void	SetUpbarColor(uint8 uColor)					{ m_nUpbarColorPartTraffic.Put(uColor); }
	//--- :xrmb ---

	// DonGato
	bool	CommunityEnabled()							{ return m_bCommunityEnabled.Get(); }
	void	SetCommunityEnabled(bool bEnable)			{ m_bCommunityEnabled.Put(bEnable); }
	CString	CommunityString()							{ return CString(m_sCommunityString.Get().c_str()); }
	void	SetCommunityString(CString strComm)			{ m_sCommunityString.Put(strComm.GetBuffer(0)); }
	bool	CommunityNoBanEnabled()						{ return m_bCommunityNoBan.Get(); }
	void	SetCommunityNoBanEnabled(bool bEnable)		{ m_bCommunityNoBan.Put(bEnable); }
	uint32 	NotificationDisplayTime()					{ return m_nNotificationDisplayTime.Get(); }
	void 	SetNotificationDisplayTime(uint32 uTime)	{ m_nNotificationDisplayTime.Put(uTime); }
	uint32 	NotificationFontSize()						{ return m_nNotificationFontSize.Get(); }
	void 	SetNotificationFontSize(uint32 uFontSize)	{ m_nNotificationFontSize.Put(uFontSize); }
	int16 	PriorityHigh()								{ return m_nPriorityHigh.Get(); }
	void 	SetPriorityHigh(int16 iPrio)				{ m_nPriorityHigh.Put(iPrio); }
	int16 	PriorityLow()								{ return m_nPriorityLow.Get(); }
	void 	SetPriorityLow(int16 iPrio)					{ m_nPriorityLow.Put(iPrio); }
	bool	RestartWaiting()							{ return m_bRestartWaiting.Get(); }
	void	SetRestartWaiting(bool bSet)				{ m_bRestartWaiting.Put(bSet); }
	bool	LogToFile()									{ return m_bLogToFile.Get(); }
	void	SetLogToFile(bool bEnable)					{ m_bLogToFile.Put(bEnable); }
	bool	TransferFullChunks()						{ return m_bTransferFullChunks.Get(); }
	void	SetTransferFullChunks(bool bEnable)			{ m_bTransferFullChunks.Put(bEnable); }
	bool	DisableXSUpTo()								{ return m_bDisableXSUpTo.Get(); }
	void	SetDisableXSUpTo(bool bDisable)				{ m_bDisableXSUpTo.Put(bDisable); }
	uint16	XSUpTo()									{ return m_nXSUpTo.Get(); }
	void	SetXSUpTo(uint16 uVal)						{ m_nXSUpTo.Put(uVal); }
	uint32  SrvConTimeout()								{ return m_nSrvConTimeout.Get(); }
	void	SetSrvConTimeout(uint32 timeout)			{ m_nSrvConTimeout.Put(timeout); }
	bool	SlowComplete()								{ return m_bSlowComplete.Get(); }
	void	SetSlowComplete(uint16 bSlow)				{ m_bSlowComplete.Put(bSlow); }
	uint16	SlowCompleteBlockSize()						{ return m_nSlowCompleteBlockSize.Get(); }
	void	SetSlowCompleteBlockSize(uint16 uSize)		{ m_nSlowCompleteBlockSize.Put(uSize); }
	CString	GetUsedFont()								{ return CString(m_sUsedFont.Get().c_str()); }
	void	SetUsedFont(CString font)					{ m_sUsedFont.Put(font.GetBuffer(0)); }
	uint8	GetFontSize()								{ return m_nFontSize.Get(); }
	void	SetFontSize(uint8 size)						{ m_nFontSize.Put(size); }
	enHashPriority	GetHashingPriority(void)			{ return m_HashingPriority.Get(); }
	void	SetHashingPriority(enHashPriority hashing)	{ m_HashingPriority.Put(hashing); }
	bool	IsUploadPartsEnabled()						{ return m_bUploadParts.Get(); }
	void	SetUploadPartsEnabled(bool bEnable)			{ m_bUploadParts.Put(bEnable); }
	bool	IsA4AFStringEnabled()						{ return m_bA4AFStringEnabled.Get(); }
	void	SetA4AFStringEnabled(bool bEnable)          { m_bA4AFStringEnabled.Put(bEnable); }
	bool	IsBanMessageEnabled()						{ return m_bBanMessageEnabled.Get(); }
	void	SetBanMessageEnabled(bool bEnable)          { m_bBanMessageEnabled.Put(bEnable); }
	int		GetMethod()									{ return m_nSearchMethod.Get(); }
	void	SetMethod(int Value) 						{ m_nSearchMethod.Put(Value); }
	bool	AutoTakeED2KLinks()							{ return m_bAutoTakeLinks.Get(); }
	void	SetAutoTakeED2KLinks(bool bEnable)          { m_bAutoTakeLinks.Put(bEnable); }
	uint16	GetFileBufferSize()							{ return m_nFileBufferSize.Get(); }
	void	SetFileBufferSize(int Value)      		    { m_nFileBufferSize.Put(Value); }
	int		GetMainProcessPriority()					{ return m_nMainProcessPriority.Get(); }
	void	SetMainProcessPriority(int Value);

	bool	IsSCHEnabled()								{ return m_bSCHEnabled.Get(); }
	void	SetSCHEnabled(bool bEnable)					{ m_bSCHEnabled.Put(bEnable); }
	bool	IsSCHExceptMon()							{ return m_bSCHExceptMon.Get(); }
	void	SetSCHExceptMon(bool bEnable)				{ m_bSCHExceptMon.Put(bEnable); }
	bool	IsSCHExceptTue()							{ return m_bSCHExceptTue.Get(); }
	void	SetSCHExceptTue(bool bEnable)				{ m_bSCHExceptTue.Put(bEnable); }
	bool	IsSCHExceptWed()							{ return m_bSCHExceptWed.Get(); }
	void	SetSCHExceptWed(bool bEnable)				{ m_bSCHExceptWed.Put(bEnable); }
	bool	IsSCHExceptThu()							{ return m_bSCHExceptThu.Get(); }
	void	SetSCHExceptThu(bool bEnable)				{ m_bSCHExceptThu.Put(bEnable); }
	bool	IsSCHExceptFri()							{ return m_bSCHExceptFri.Get(); }
	void	SetSCHExceptFri(bool bEnable)				{ m_bSCHExceptFri.Put(bEnable); }
	bool	IsSCHExceptSat()							{ return m_bSCHExceptSat.Get(); }
	void	SetSCHExceptSat(bool bEnable)				{ m_bSCHExceptSat.Put(bEnable); }
	bool	IsSCHExceptSun()							{ return m_bSCHExceptSun.Get(); }
	void	SetSCHExceptSun(bool bEnable)				{ m_bSCHExceptSun.Put(bEnable); }
	uint32	GetSCHShift1()								{ return m_nSCHShift1.Get(); }
	void	SetSCHShift1(uint32 Time)					{ m_nSCHShift1.Put(Time); }
	uint32	GetSCHShift2()								{ return m_nSCHShift2.Get(); }
	void	SetSCHShift2(uint32 Time)					{ m_nSCHShift2.Put(Time); }
	uint16	GetSCHShift1Upload()						{ return m_nSCHShift1Upload.Get(); }
	void	SetSCHShift1Upload(uint16 Value)			{ m_nSCHShift1Upload.Put(Value); }
	uint16	GetSCHShift1Download()						{ return m_nSCHShift1Download.Get(); }
	void	SetSCHShift1Download(uint16 Value)			{ m_nSCHShift1Download.Put(Value); }
	uint16	GetSCHShift1conn()							{ return m_nSCHShift1conn.Get(); }
	void	SetSCHShift1conn(uint16 Value)				{ m_nSCHShift1conn.Put(Value); }
	uint16	GetSCHShift15sec()							{ return m_nSCHShift15sec.Get(); }
	void	SetSCHShift15sec(uint16 Value)				{ m_nSCHShift15sec.Put(Value); }
	uint16	GetSCHShift2Upload()						{ return m_nSCHShift2Upload.Get(); }
	void	SetSCHShift2Upload(uint16 Value)			{ m_nSCHShift2Upload.Put(Value); }
	uint16	GetSCHShift2Download()						{ return m_nSCHShift2Download.Get(); }
	void	SetSCHShift2Download(uint16 Value)			{ m_nSCHShift2Download.Put(Value); }
	uint16	GetSCHShift2conn()							{ return m_nSCHShift2conn.Get(); }
	void	SetSCHShift2conn(uint16 Value)				{ m_nSCHShift2conn.Put(Value); }
	uint16	GetSCHShift25sec()							{ return m_nSCHShift25sec.Get(); }
	void	SetSCHShift25sec(uint16 Value)				{ m_nSCHShift25sec.Put(Value); }

	// quick-speed changer [xrmb]
	void	SetMaxUpload(uint16 nMaxUpload) 			{ m_nMaxUpload.Put(nMaxUpload); }
	void	SetMaxDownload(uint16 nMaxDownload)			{ m_nMaxDownload.Put(nMaxDownload); }
	uint16  GetMaxDownloadValue()						{ return m_nMaxDownload.Get(); }

	// LANCAST (moosetea) - Get/Set Properties
	bool GetLancastEnabled()							{ return m_LancastEnabled.Get(); }
	void SetLancastEnabled(bool bEnable)				{ m_LancastEnabled.Put(bEnable); }
	bool GetLancastUseDefaultIP()						{ return m_UseDefaultIP.Get(); }
	uint32 GetLancastIP();
	uint32 GetLancastSubnet()							{ return m_LancastSubnet.Get(); }
	uint16 GetLancastPort()								{ return m_LancastPort.Get(); }

	WINDOWPLACEMENT GetEmuleWindowPlacement()			{ return prefs->EmuleWindowPlacement; }
	void SetWindowLayout(WINDOWPLACEMENT in) 			{ prefs->EmuleWindowPlacement = in; }
	static int	GetRecommendedMaxConnections();

	void	LoadServerlistAddresses() ; // Tarod (read addresses.dat every time)

	// Maverick
	bool	GetMlDonkeyCM()								{ return m_bMlDonkeyCM.Get(); }
	void	SetMlDonkeyCM(bool bEnable)					{ m_bMlDonkeyCM.Put(bEnable); }
	ProxySettings GetProxy()                            { ProxySettings Ret = prefs->proxy; return Ret; }
	void SetProxySettings(ProxySettings proxysettings)  { prefs->proxy = proxysettings; }

	CStringList shareddir_list;
	CStringList tempdir_list;	// InterCeptor (multiple tempdirs)
	CStringList adresses_list;

	void SetLanguage();
	int	GetIPFilterLevel()								{ return 128; }
	CString GetMessageFilter()							{ return prefs->messageFilter; }
	void SetMessageFilter(CString in)					{ prefs->messageFilter=in; }
	bool	ShowRatesOnTitle()							{ return prefs->showRatesInTitle; }
	void	SetRatesOnTitle(bool show) 					{ prefs->showRatesInTitle=show; }

	//START enkeyDEV(kei-kun) -Messaging preferences-
	CString   GetAwayStateMessage()						{ return CString(m_sAwayStateMessage.Get().c_str()); }
	void	SetAwayStateMessage(CString in)				{ m_sAwayStateMessage.Put(in.GetBuffer(0)); }
	uint8   GetAcceptMessagesFrom()						{ return m_nAcceptMessagesFrom.Get(); }
	void	SetAcceptMessagesFrom(uint8 in)				{ m_nAcceptMessagesFrom.Put(in); }
	bool    GetAwayState()								{ return m_nAwayState.Get(); }
	void	SetAwayState(bool in)						{ m_nAwayState.Put(in); }
	//END enkeyDEV(kei-kun) -Messaging preferences-
	uint16	GetMsgSessionsMax()						{ return prefs->maxmsgsessions;}

	void	SetUseProxy(bool in)		{ prefs->proxy.UseProxy=in;}
	bool	IsProxyASCWOP()				{ return prefs->m_bIsASCWOP;}
	void	SetProxyASCWOP(bool in)		{ prefs->m_bIsASCWOP=in;}


protected:
	void		CreateUserHash();
	void		SetStandartValues();
private:
	char* appdir;
	Preferences_Struct_DONTUSE* prefs;
	Preferences_Ext_Struct* prefsExt;

	Preferences_Import19c_Struct* prefsImport19c;
	Preferences_Import20a_Struct* prefsImport20a;
	Preferences_Import20b_Struct* prefsImport20b;

	uchar userhash[16];
	WORD  m_wWinVer;

	void LoadPreferences();
	void SavePreferences();

private:
	CSecuredVar<string>		m_sNick;
	CSecuredVar<WORD>		m_nLanguageID;

	CSecuredVar<uint16>		m_nMaxUpload;
	CSecuredVar<uint16>		m_nMaxDownload;
	CSecuredVar<uint16>		m_nMaxConnections;
	CSecuredVar<uint16>		m_nMaxSourcePerFile;

	CSecuredVar<uint64>		m_nTotalDownloadedBytes;
	CSecuredVar<uint64>		m_nTotalUploadedBytes;

	CSecuredVar<int>		m_nMaxGraphDownloadRate;
	CSecuredVar<int>		m_nMaxGraphUploadRate;
	CSecuredVar<uint8>		m_nGraphRatio;

	CSecuredVar<bool>		m_bReconnect;
	CSecuredVar<int8>		m_nDeadServer;
	CSecuredVar<int8>		m_nScoreSystem;
	CSecuredVar<bool>		m_bAutoServerList;
	CSecuredVar<bool>		m_bAutoConnect;
	CSecuredVar<bool>		m_bAutoConnectStaticOnly;
	CSecuredVar<bool>		m_bAddServersFromConn;
	CSecuredVar<bool>		m_bFilterBadIP;
	CSecuredVar<enSeeShare>	m_nSeeShares;

	CSecuredVar<bool>		m_bUpdateNotify;
	CSecuredVar<bool>		m_bMinToTray;
	CSecuredVar<bool>		m_bConfirmExit;
	CSecuredVar<bool>		m_bBeepOnError;
	CSecuredVar<bool>		m_bSplashScreen;
	CSecuredVar<bool>		m_bOnlineSig;
	CSecuredVar<bool>		m_bShowOverhead;
	CSecuredVar<bool>		m_bBackupPreview;
	CSecuredVar<int8>		m_nTransferDoubleclick;

	CSecuredVar<int>		m_nDepth3D;
	CSecuredVar<uint16>		m_nTrafficOMeterInterval;
	CSecuredVar<uint16>		m_nStatsInterval;
	CSecuredVar<uint16>		m_nStatsColumnWidth;
	CSecuredVar<uint16>		m_nDetailColumnWidth;		//Cax2 
	CSecuredVar<string>		m_sStatsListExpanded;

	CSecuredVar<string>		m_sIncomingDir;
	CSecuredVar<string>		m_sTempDir;
	CSecuredVar<string>		m_sVlcDir;
	CSecuredVar<string>		m_sVideoPlayer;		//Cax2 import from official client.

	CSecuredVar<uint16>		m_nPort;
	CSecuredVar<uint16>		m_nUDPPort;
	CSecuredVar<bool>		m_bUseRQlimit;	//Cax2 change
	CSecuredVar<uint16>		m_nRQlimit;

	CSecuredVar<bool>		m_bA4AFStringEnabled;
	CSecuredVar<bool>		m_bBanMessageEnabled;
	CSecuredVar<int>		m_nSearchMethod;
	CSecuredVar<bool>		m_bAutoTakeLinks;
	CSecuredVar<uint16>		m_nFileBufferSize;
	CSecuredVar<int>		m_nMainProcessPriority;

	// Scheduler
	CSecuredVar<bool>		m_bSCHEnabled;
	CSecuredVar<uint32>		m_nSCHShift1;
	CSecuredVar<uint32>		m_nSCHShift2;
	CSecuredVar<uint16>		m_nSCHShift1Upload;
	CSecuredVar<uint16>		m_nSCHShift1Download;
	CSecuredVar<uint16>		m_nSCHShift1conn;
	CSecuredVar<uint16>		m_nSCHShift15sec;
	CSecuredVar<uint16>		m_nSCHShift2Upload;
	CSecuredVar<uint16>		m_nSCHShift2Download;
	CSecuredVar<uint16>		m_nSCHShift2conn;
	CSecuredVar<uint16>		m_nSCHShift25sec;
	CSecuredVar<bool>		m_bSCHExceptMon;
	CSecuredVar<bool>		m_bSCHExceptTue;
	CSecuredVar<bool>		m_bSCHExceptWed;
	CSecuredVar<bool>		m_bSCHExceptThu;
	CSecuredVar<bool>		m_bSCHExceptFri;
	CSecuredVar<bool>		m_bSCHExceptSat;
	CSecuredVar<bool>		m_bSCHExceptSun;

	CSecuredVar<uint16>		m_nProfile;
	CSecuredVar<bool>		m_bUAP;
	CSecuredVar<bool>		m_bDAP;
	CSecuredVar<bool>		m_bDisableXS;
	CSecuredVar<bool>		m_bStartDownloadPaused;
	CSecuredVar<bool>		m_bDownloadPausedOnComplete;
	CSecuredVar<bool>		m_bRemoveNNS;

	CSecuredVar<uint32>		m_nBadClientBanTime;
	CSecuredVar<uint32>		m_nMinRequestTime;
	CSecuredVar<uint8>		m_nBadClientBan;

	CSecuredVar<string>		m_sToolbarSettings;

	CSecuredVar<bool>		m_bUseProxyListenPort;
	CSecuredVar<uint16>		m_nListenPort;
	CSecuredVar<bool>		m_bMlDonkeyCM;

	CSecuredVar<uint32>		m_nSLSRestoreSourcesTime;
	CSecuredVar<uint32>		m_nSLSLoadQuantities;
	CSecuredVar<uint32>		m_nSLSOutdated;
	CSecuredVar<bool>		m_bSLSEnable;

	CSecuredVar<int16>		m_nQueueSize;
	CSecuredVar<uint16>		m_nDeadServerRetries;
	CSecuredVar<bool>		m_bSmartIdCheck;
	CSecuredVar<uint8>		m_nSmartIdState;
	CSecuredVar<bool>		m_bVerbose;
	CSecuredVar<bool>		m_bUpdateQueueList;
	CSecuredVar<uint16>		m_nMaxConperFive;
	CSecuredVar<bool>		m_bSafeConnect;

	CSecuredVar<bool>		m_bMultiple;
	CSecuredVar<int8>		m_nTooltipDelayTime;
	CSecuredVar<bool>		m_bBringToForeground;
	CSecuredVar<int8>		m_nSplitterBarPosition;
	CSecuredVar<uint16>		m_nStatsMax;
	CSecuredVar<int8>		m_nStatsAverageMinutes;
	CSecuredVar<bool>		m_bShowToolbarSpeedMeter; 
	CSecuredVar<bool>		m_bCloseToTray;
	CSecuredVar<bool>		m_bDetailsOnClick;
	CSecuredVar<bool>		m_bStartMinimized;
	CSecuredVar<bool>		m_bCommunityEnabled;
	CSecuredVar<string>		m_sCommunityString;
	CSecuredVar<bool>		m_bCommunityNoBan;
	CSecuredVar<int16>		m_nPriorityHigh;
	CSecuredVar<int16>		m_nPriorityLow;
	CSecuredVar<bool>		m_bRestartWaiting;
	CSecuredVar<bool>		m_bLogToFile;
	CSecuredVar<bool>		m_bTransferFullChunks;
	CSecuredVar<bool>		m_bDisableXSUpTo;
	CSecuredVar<uint16>		m_nXSUpTo;
	CSecuredVar<uint32>		m_nSrvConTimeout;
	CSecuredVar<bool>		m_bSlowComplete;
	CSecuredVar<uint16>		m_nSlowCompleteBlockSize;
	CSecuredVar<enHashPriority>	m_HashingPriority;
	CSecuredVar<bool>		m_bUploadParts;

	CSecuredVar<bool>		m_bUseDownloadNotifier;
	CSecuredVar<bool>		m_bUseDownloadAddNotifier;
	CSecuredVar<bool>		m_bUseChatNotifier;
	CSecuredVar<bool>		m_bUseLogNotifier;	
	CSecuredVar<bool>		m_bUseSoundInNotifier;
	CSecuredVar<bool>		m_bNotifierImportantError;
	CSecuredVar<string>		m_sNotifierSoundFilePath;
	CSecuredVar<bool>		m_bNotifierPopsEveryChatMsg;
	CSecuredVar<uint32>		m_nNotificationDisplayTime;
	CSecuredVar<uint32>		m_nNotificationFontSize;

	CSecuredVar<string>		m_sUsedFont;
	CSecuredVar<uint8>		m_nFontSize;

	// Sort
	CSecuredVar<DWORD>		m_nStatColors[15];
	CSecuredVar<bool>		m_bUseSort;
	CSecuredVar<bool>		m_bUseSrcSortCol2;
	CSecuredVar<int8>		m_nServerSortCol;
	CSecuredVar<int8>		m_nUploadSortCol;
	CSecuredVar<int8>		m_nQueueSortCol;
	CSecuredVar<int8>		m_nSearchSortCol;
	CSecuredVar<int8>		m_nIRCSortCol;
	CSecuredVar<uint16>		m_nFileSortCol;
	CSecuredVar<int8>		m_nDownloadSortCol;
	CSecuredVar<int8>		m_nSrcSortCol1;
	CSecuredVar<int8>		m_nSrcSortCol2;

	// IRC
	CSecuredVar<string>		m_sIRCNick;
	CSecuredVar<string>		m_sIRCServer;
	CSecuredVar<bool>		m_bIRCAddTimestamp;
	CSecuredVar<string>		m_sIRCChanNameFilter;
	CSecuredVar<bool>		m_bIRCUseChanFilter;
	CSecuredVar<uint16>		m_nIRCChannelUserFilter;
	CSecuredVar<string>		m_sIRCPerformString;
	CSecuredVar<bool>		m_bIRCUsePerform;
	CSecuredVar<bool>		m_bIRCListOnConnect;
	CSecuredVar<bool>		m_bIRCAcceptLinks;
	CSecuredVar<bool>		m_bIRCIgnoreInfoMessage;
	CSecuredVar<bool>		m_bIRCIgnoreEmuleProtoInfoMessage;

	// Web Server
	CSecuredVar<string>		m_sWebPassword;
	CSecuredVar<string>		m_sLowWebPassword;
	CSecuredVar<uint16>		m_nWebPort;
	CSecuredVar<bool>		m_bWebEnabled;
	CSecuredVar<bool>		m_bWebLowEnabled;
	CSecuredVar<bool>		m_bWebUseGzip;
	CSecuredVar<int>		m_nWebPageRefresh;

	// Part Traffic
	CSecuredVar<bool>		m_bUsePartTraffic;
	CSecuredVar<uint8>		m_nUpbarPartTraffic;		// 0=one info 1=2 infos 2=3infos
	CSecuredVar<uint8>		m_nUpbarColorPartTraffic;	// 0=rainbow style 1=more rainbow 2=downloadlist-like 3=girlie

	// LANCAST (moosetea) - Settings
	CSecuredVar<bool>  		m_LancastEnabled;
	CSecuredVar<bool>   	m_UseDefaultIP;
	CSecuredVar<uint32> 	m_DefaultLancastIP;
	CSecuredVar<uint32>		m_LancastIP;
	CSecuredVar<uint32>		m_LancastSubnet;
	CSecuredVar<uint16>		m_LancastPort;


	CSecuredVar<string>		m_sToolbarBitmap;
	CSecuredVar<string>		m_sToolbarBitmapFolder;
	CSecuredVar<int8>		m_nToolbarLabels;

	//START enkeyDEV(kei-kun) -Messaging preferences-
	CSecuredVar<uint8>		m_nAcceptMessagesFrom;
	CSecuredVar<int8>		m_nAwayState;
	CSecuredVar<string>		m_sAwayStateMessage;

	CSecuredVar<BOOL>		m_bIsNTBased;

	static bool bWinVerAlreadyDetected;

public:
	BOOL IsNTBased(void);
};
